//	Zinc Application Framework - Z_BNUM1.HPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#if !defined(Z_BNUM1_HPP)
#define Z_BNUM1_HPP
#include <z_bnum.hpp>
#include <z_str1.hpp>

// --------------------------------------------------------------------------
// ----- ZafBignum ----------------------------------------------------------
// --------------------------------------------------------------------------

class ZAF_EXPORT ZafBignum : public ZafString
{
public:
	// --- General members ---
	ZafBignum(int left, int top, int width,
		ZafBignumData *bignumData = ZAF_NULLP(ZafBignumData));
	ZafBignum(int left, int top, int width, long value);
	ZafBignum(int left, int top, int width, double value);
	ZafBignum(const ZafBignum &copy);
	virtual ~ZafBignum(void);

	// --- Persist members ---
	virtual ZafWindowObject *Duplicate(void);
	ZafBignum(const ZafIChar *name, ZafObjectPersistence &persist);
	static ZafElement *Read(const ZafIChar *name, ZafObjectPersistence &persist);
	virtual void Write(ZafObjectPersistence &persist);

	// --- Attributes & data access ---
	// get
	ZafBignumData *BignumData(void) const;
	long IValue(void);
	double RValue(void);
	// set
	virtual ZafError SetBignum(long value);
	virtual ZafError SetBignum(double value);
	virtual ZafError SetBignumData(ZafBignumData *bignumData);

	// --- Data access ---
	virtual ZafError Decrement(ZafData *data, bool wrapData = false);
	virtual ZafError Increment(ZafData *data, bool wrapData = false);
	virtual ZafError Validate(bool processError = true);

	// --- String members ---
	virtual bool SetLowerCase(bool lowerCase);
	virtual bool SetPassword(bool password);
	virtual ZafError SetStringData(ZafStringData *string);
	virtual bool SetUpperCase(bool upperCase);
	virtual bool SetVariableName(bool variableName);

	// --- Window object members ---
	virtual ZafEventType Event(const ZafEventStruct &event);
	virtual ZafError SetText(const ZafIChar *text);

	// --- Class identification ---
	static ZafClassID classID;
	static ZafClassNameChar ZAF_FARDATA className[];
	virtual ZafClassID ClassID(void) const { return (classID); }
	virtual ZafClassName ClassName(void) const { return (className); }
	virtual bool IsA(ZafClassID compareID) const;
	virtual bool IsA(ZafClassName compareName) const;

protected:
	friend class ZAF_EXPORT OSBignum;

	// --- General members ---
	ZafBignumData *bignumData;

	// --- Data access ---
	static ZafError Update(ZafBignum *bignum, ZafUpdateType type)
		{ if (type == ZAF_UPDATE_OBJECT) return bignum->OSSetBignum();
		  else if (type == ZAF_UPDATE_DATA) return bignum->OSGetBignum();
		  else return (ZAF_ERROR_INVALID); }

	// --- Persist members ---
	void ReadData(ZafObjectPersistence &persist);
	virtual ZafObjectConstructor ReadFunction(void) { return (ZafBignum::Read); }
	void WriteData(ZafObjectPersistence &persist);

	// --- Special OS members ---
	ZafError OSGetBignum(void);
	ZafError OSSetBignum(void);

	// --- Language & locale support ---
	ZafLanguageData *messages;
};

// --- inline functions ---
inline ZafBignumData *ZafBignum::BignumData(void) const { return (bignumData); }
inline ZafError ZafBignum::SetText(const ZafIChar *text) { return (bignumData->SetBignum(text, inputFormatData ? inputFormatData->Text() : ZAF_NULLP(ZafIChar))); }

#endif // Z_BNUM1_HPP

