//	Zinc Application Framework - Z_HLIST.HPP
//	COPYRIGHT (C) 1990-1998.  All Rights Reserved.
//	Zinc Software Incorporated.  Pleasant Grove, Utah  USA

#if !defined(Z_HLIST_HPP)
#define Z_HLIST_HPP
#include <z_win.hpp>

// --------------------------------------------------------------------------
// ----- ZafHzList ----------------------------------------------------------
// --------------------------------------------------------------------------

class ZAF_EXPORT ZafHzList : public ZafWindow
{
public:
	// --- General class members ---
	ZafHzList(int left, int top, int width, int height, int cellWidth,
		int cellHeight);
	ZafHzList(const ZafHzList &copy);
	virtual ~ZafHzList(void);

	// --- Persistent members ---
	virtual ZafWindowObject *Duplicate(void);
	ZafHzList(const ZafIChar *name, ZafObjectPersistence &persist);
	static ZafElement *Read(const ZafIChar *name, ZafObjectPersistence &persist);
	virtual void Write(ZafObjectPersistence &persist);

	// --- Attributes & data access ---
	// get
	bool AutoSortData(void) const;
	int CellHeight(void) const;
	int CellWidth(void) const;
	// set
	virtual bool SetAutoSortData(bool autoSortData);
	virtual int SetCellHeight(int cellHeight);
	virtual int SetCellWidth(int cellWidth);

	// --- Window members ---
	virtual bool SetDestroyable(bool destroyable);
	virtual bool SetLocked(bool locked);
	virtual bool SetMaximized(bool maximized);
	virtual bool SetMinimized(bool minimized);
	virtual bool SetModal(bool modal);
	virtual bool SetMoveable(bool moveable);
	virtual bool SetNormalHotKeys(bool normalHotKeys);
	virtual ZafWindow *SetOwner(ZafWindow *owner);
	virtual bool SetSizeable(bool sizeable);

	// --- Window object members ---
	virtual ZafEventType Event(const ZafEventStruct &event);
	ZafLogicalEvent LogicalEvent(const ZafEventStruct &event);
	virtual ZafError OSUpdatePalettes(ZafPaletteType zafTypes, ZafPaletteType osTypes);
	virtual bool SetAutomaticUpdate(bool automaticUpdate);
	virtual ZafLogicalColor SetBackgroundColor(ZafLogicalColor color, ZafLogicalColor mono = ZAF_MONO_NULL);
	virtual bool SetCopyDraggable(bool copyDraggable);
	virtual ZafLogicalFont SetFont(ZafLogicalFont font);
	virtual bool SetLinkDraggable(bool linkDraggable);
	virtual bool SetMoveDraggable(bool moveDraggable);
	virtual ZafLogicalColor SetTextColor(ZafLogicalColor color, ZafLogicalColor mono = ZAF_MONO_NULL);
	virtual bool SetVisible(bool visible);

	// --- Advanced event handling ---
#	if defined(ZAF_MSDOS) || defined(ZAF_CURSES)
	virtual ZafEventType DispatchMouseEvent(const ZafEventStruct &event);
	virtual ZafWindowObject *FindMouseObject(const ZafEventStruct &event);
#	endif

	// --- Class identification ---
	static ZafClassID classID;
	static ZafClassNameChar ZAF_FARDATA className[];
	virtual ZafClassID ClassID(void) const { return (classID); }
	virtual ZafClassName ClassName(void) const { return (className); }
	virtual bool IsA(ZafClassID compareID) const;
	virtual bool IsA(ZafClassName compareName) const;

	// --- List members ---
	virtual ZafWindowObject *Add(ZafWindowObject *object, ZafWindowObject *position = ZAF_NULLP(ZafWindowObject));
	virtual void Destroy(void);
	virtual void Sort(void);
	virtual ZafWindowObject *Subtract(ZafWindowObject *object);

	ZafHzList &operator+(ZafWindowObject *object) { Add(object); return(*this); }
	ZafHzList &operator-(ZafWindowObject *object) { Subtract(object); return(*this); }

	// --- Special OS members ---
	virtual OSWindowID OSScreenID(ZafScreenIDType type = ZAF_SCREENID) const;

protected:
	friend class ZAF_EXPORT OSHzList;

	// --- General members ---
	bool autoSortData;
	int cellWidth, cellHeight;

	virtual ZafRegionStruct ConvertToDrawRegion(const ZafWindowObject *object,
		const ZafRegionStruct *zafRegion = ZAF_NULLP(ZafRegionStruct)) const;
	virtual ZafEventType DragDropEvent(const ZafEventStruct &event);
	virtual ZafEventType DrawFocus(ZafRegionStruct &region, ZafEventType ccode);
	virtual ZafEventType Draw(const ZafEventStruct &event, ZafEventType ccode);
	virtual const ZafPaletteStruct *MapClassPalette(ZafPaletteType type, ZafPaletteState state);
	virtual ZafEventType MoveEvent(const ZafEventStruct &event);
	virtual ZafWindowObject *NotifyFocus(ZafWindowObject *object, bool focus);
	virtual ZafWindowObject *NotifySelection(ZafWindowObject *object, bool selected);
	virtual ZafEventType ScrollEvent(const ZafEventStruct &event);

	// --- Special OS members ---
	virtual void OSDestroy(void);
	virtual ZafEventType OSEvent(const ZafEventStruct &event);
	virtual void OSMapPalette(ZafPaletteStruct &palette, ZafPaletteType type, ZafPaletteState state);
	virtual void OSSize(void);
	virtual void OSSort(void);
	virtual void OSRegisterObject(void);

	// --- Persistent members ---
	void ReadData(ZafObjectPersistence &persist);
	virtual ZafObjectConstructor ReadFunction(void) { return (ZafHzList::Read); }
	void WriteData(ZafObjectPersistence &persist);

private:
	static ZafPaletteMap ZAF_FARDATA defaultPaletteMap[];
	static ZafEventMap ZAF_FARDATA defaultEventMap[];

#	if defined(ZAF_MOTIF)
	int itemWidth, itemHeight;
	ZafWindowObject *firstVisible, *lastVisible, *cancelObject;
	void CheckMembers(ZafWindowObject *item);
	void ComputeNextPosition(ZafWindowObject *object, int &column, int &line);
	void ComputePreviousPosition(ZafWindowObject *object, int &column, int &line);
#	elif defined(ZAF_MSDOS) || defined(ZAF_CURSES)
	ZafWindowObject *firstVisible, *lastVisible;
#	endif
};

// --- inline functions ---
inline bool ZafHzList::AutoSortData(void) const { return (autoSortData); }
inline int ZafHzList::CellHeight(void) const { return (cellHeight); }
inline int ZafHzList::CellWidth(void) const { return (cellWidth); }

#endif // Z_HLIST_HPP

